/*******************************************************************************
 * Copyright 2012 pw999
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package net.sf.oqt.jpa;

import static org.fest.assertions.Assertions.assertThat;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import net.sf.oqt.core.CoreFactory;
import net.sf.oqt.jpa.EntityFinder;
import net.sf.oqt.jpa.testentities.CrossTable;
import net.sf.oqt.jpa.testentities.TableA;
import net.sf.oqt.jpa.testentities.TestInfo;
import net.sf.oqt.jpa.testentities.genericpackage.AbstractTableA;
import net.sf.oqt.jpa.testentities.genericpackage.ImplementingTableA1;
import net.sf.oqt.jpa.testentities.subpackage.RefA;
import net.sf.oqt.model.EntityVO;
import net.sf.oqt.model.PackageVO;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import edu.emory.mathcs.backport.java.util.Arrays;

public class EntityFinderTest {

	@Before
	public void setUp() throws Exception {
		CoreFactory.getProperties().getPackageNames().clear();
		CoreFactory.getProperties().setUrl(TestInfo.URL_H2);
		CoreFactory.getProperties().setDriverName(TestInfo.DRIVER_H2);
	}

	@After
	public void tearDown() throws Exception {
		CoreFactory.shutdown();
	}

	/**
	 * Mostly targeted at testing the number of entities and the number of
	 * queries found.
	 * 
	 * @throws ClassNotFoundException
	 *             if any
	 */
	@Test
	public void testSubPackage() throws ClassNotFoundException {
		final Set<String> basePackages = new HashSet<String>(1);
		basePackages.add(RefA.class.getPackage().getName());
		final Set<PackageVO> packages = EntityFinder.findAllEntities(basePackages);
		assertThat(packages).isNotNull().isNotEmpty().hasSize(1);
		final PackageVO p = packages.iterator().next();
		assertThat(p).isNotNull().isEqualTo(new PackageVO(RefA.class.getPackage().getName()));
		assertThat(p.getEntities()).isNotNull().isNotEmpty().hasSize(TestInfo.numberOfEntitiesInSubPackage);
		final EntityVO e1 = packages.iterator().next().getEntities().get(0);
		final EntityVO e2 = packages.iterator().next().getEntities().get(1);
		assertThat(e1).isNotNull().isEqualTo(new EntityVO("RefA"));
		assertThat(e2).isNotNull().isEqualTo(new EntityVO("RefB"));

		assertThat(e1.getQueries()).isNotNull().isNotEmpty().hasSize(1);
		assertThat(e2.getQueries()).isNotNull().isNotEmpty().hasSize(2);

		assertThat(e1.getQueries().iterator().next().getName()).isNotNull().isEqualTo("findByReferenceADescription");
		assertThat(e1.getQueries().iterator().next().getJPQLQuery()).isNotNull().isEqualTo("select ent from RefA ent where ent.description like :descr");

		assertThat(e2.getQueries().iterator().next().getName()).isNotNull().isEqualTo("getRefBByCode");
		assertThat(e2.getQueries().iterator().next().getJPQLQuery()).isNotNull().isEqualTo("select ent from RefB ent where ent.code = :code");
	}

	@Test
	public void testMainPackages() throws ClassNotFoundException {
		final Set<String> basePackages = new HashSet<String>(1);
		basePackages.add(CrossTable.class.getPackage().getName());
		final Set<PackageVO> packages = EntityFinder.findAllEntities(basePackages);
		assertThat(packages).isNotNull().isNotEmpty().hasSize(3);
		// FEST assert contains() method doesn't realy work very well
		assertThat(packages.containsAll(Arrays.asList(new PackageVO[] { new PackageVO(RefA.class.getPackage().getName()), new PackageVO(CrossTable.class.getPackage().getName()), new PackageVO(ImplementingTableA1.class.getPackage().getName()) }))).isEqualTo(true);
		final Iterator<PackageVO> it = packages.iterator();
		PackageVO p = it.next();
		assertThat(p.getName()).isNotNull().isNotEmpty().isEqualTo(TableA.class.getPackage().getName());
		assertThat(p.getEntities()).isNotNull().isNotEmpty().hasSize(TestInfo.numberOfEntities - TestInfo.numberOfEntitiesInGenericPackage - TestInfo.numberOfEntitiesInSubPackage);
		p = it.next();
		assertThat(p.getName()).isNotNull().isNotEmpty().isEqualTo(AbstractTableA.class.getPackage().getName());
		assertThat(p.getEntities()).isNotNull().isNotEmpty().hasSize(TestInfo.numberOfEntitiesInGenericPackage);
		p = it.next();
		assertThat(p.getName()).isNotNull().isNotEmpty().isEqualTo(RefA.class.getPackage().getName());
		assertThat(p.getEntities()).isNotNull().isNotEmpty().hasSize(TestInfo.numberOfEntitiesInSubPackage);

	}

	@Test
	public void testMainPackagesWithExtraSubPackageDeclared() throws ClassNotFoundException {
		final Set<String> basePackages = new HashSet<String>(4);
		basePackages.add(RefA.class.getPackage().getName());
		basePackages.add(CrossTable.class.getPackage().getName());
		basePackages.add(ImplementingTableA1.class.getPackage().getName());
		basePackages.add(CoreFactory.class.getPackage().getName());
		final Set<PackageVO> packages = EntityFinder.findAllEntities(basePackages);
		assertThat(packages).isNotNull().isNotEmpty().hasSize(3);
		// FEST assert contains() method doesn't realy work very well
		assertThat(packages.containsAll(Arrays.asList(new PackageVO[] { new PackageVO(RefA.class.getPackage().getName()), new PackageVO(CrossTable.class.getPackage().getName()), new PackageVO(ImplementingTableA1.class.getPackage().getName()) }))).isEqualTo(true);
		final Iterator<PackageVO> it = packages.iterator();
		PackageVO p = it.next();
		assertThat(p.getName()).isNotNull().isNotEmpty().isEqualTo(TableA.class.getPackage().getName());
        assertThat(p.getEntities()).isNotNull().isNotEmpty().hasSize(TestInfo.numberOfEntities - TestInfo.numberOfEntitiesInGenericPackage - TestInfo.numberOfEntitiesInSubPackage);
        p = it.next();
        assertThat(p.getName()).isNotNull().isNotEmpty().isEqualTo(AbstractTableA.class.getPackage().getName());
        assertThat(p.getEntities()).isNotNull().isNotEmpty().hasSize(TestInfo.numberOfEntitiesInGenericPackage);
        p = it.next();
        assertThat(p.getName()).isNotNull().isNotEmpty().isEqualTo(RefA.class.getPackage().getName());
        assertThat(p.getEntities()).isNotNull().isNotEmpty().hasSize(TestInfo.numberOfEntitiesInSubPackage);

	}
}
