/*******************************************************************************
 * Copyright 2012 pw999
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ******************************************************************************/
package net.sf.oqt.jpa;

import static org.fest.assertions.Assertions.assertThat;

import java.sql.SQLException;
import java.util.Collection;
import java.util.Set;
import java.util.TreeSet;

import net.sf.oqt.core.CoreFactory;
import net.sf.oqt.jpa.EntityFinder;
import net.sf.oqt.jpa.JPATransformer;
import net.sf.oqt.jpa.testentities.CrossTable;
import net.sf.oqt.jpa.testentities.TestInfo;
import net.sf.oqt.model.EntityVO;
import net.sf.oqt.model.PackageVO;
import net.sf.oqt.model.QueryVO;
import net.sf.oqt.test.VersionSupport;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;


public class JPATransformerTest {
	Set<String> packageNames = null;
	Set<PackageVO> packages = null;

	@Before
	public void setUp() throws Exception {
		packageNames = new TreeSet<String>();
		packageNames.add(CrossTable.class.getPackage().getName());
		packages = EntityFinder.findAllEntities(packageNames);
		for (final PackageVO packageVO : packages) {
			final Collection<EntityVO> e = packageVO.getEntities();
			for (final EntityVO entityVO : e) {
				CoreFactory.getProperties().getFqns().add(packageVO.getName() + "." + entityVO.getName());
			}
		}
	}

	@After
	public void tearDown() throws Exception {
		CoreFactory.shutdown();
	}

	private void setupH2() {
		CoreFactory.getProperties().setUrl(TestInfo.URL_H2);
		CoreFactory.getProperties().setUsername("");
		CoreFactory.getProperties().setPassword("");
		CoreFactory.getProperties().setDriverName(TestInfo.DRIVER_H2);
		CoreFactory.getProperties().setDictionary(TestInfo.DICTIONARY_H2);
	}

	private void setupDerby() {
		CoreFactory.getProperties().setUrl(TestInfo.URL_DERBY);
		CoreFactory.getProperties().setUsername("");
		CoreFactory.getProperties().setPassword("");
		CoreFactory.getProperties().setDriverName(TestInfo.DRIVER_DERBY);
		CoreFactory.getProperties().setDictionary(TestInfo.DICTIONARY_DERBY);
	}

	private void setupHSQL() {
		CoreFactory.getProperties().setUrl(TestInfo.URL_HSQL);
		CoreFactory.getProperties().setUsername("sa");
		CoreFactory.getProperties().setPassword("");
		CoreFactory.getProperties().setDriverName(TestInfo.DRIVER_HSQL);
		CoreFactory.getProperties().setDictionary(TestInfo.DICTIONARY_HSQL);
	}

	@Test
	public void testCase() throws ClassNotFoundException, InstantiationException, IllegalAccessException, SQLException {
		setupH2();
		if (VersionSupport.isNotSupported(CoreFactory.getProperties())) {
			return;
		}
		JPATransformer.transformAllQueries(packages);
		assertThat(packages.iterator().next().getEntities().get(0).getQueries().iterator().next().getSQLQuery()).isNotNull().isNotEmpty();
	}

	@Test
	public void testSimpleSelectH2() throws Exception {
		setupH2();
		if (VersionSupport.isNotSupported(CoreFactory.getProperties())) {
			return;
		}
		final QueryVO q = new QueryVO("name", "select ent from TableA ent");
		JPATransformer.transform(q);
		assertThat(q.getError()).isNull();
		assertThat(q.getSQLQuery()).isNotNull().isNotEmpty().hasSize(1);
		assertThat(CoreFactory.getQueries()).isNotEmpty().hasSize(1);
		assertThat(q.getSQLQuery().iterator().next()).isEqualToIgnoringCase("SELECT t0.ID, t0.AGE, t0.COMMENT, t0.FAV, t0.NAME, t0.SURNAME FROM TABLE_A t0");
	}

	@Test
	public void testWithParamH2() throws Exception {
		setupH2();
		if (VersionSupport.isNotSupported(CoreFactory.getProperties())) {
			return;
		}
		final QueryVO q = new QueryVO("name", "select ent from TableA ent where ent.id = :id");
		JPATransformer.transform(q);
		assertThat(q.getError()).isNull();
		assertThat(q.getSQLQuery()).isNotNull().isNotEmpty().hasSize(1);
		assertThat(CoreFactory.getQueries()).isNotEmpty().hasSize(1);
		assertThat(q.getSQLQuery().iterator().next()).isEqualToIgnoringCase("SELECT t0.ID, t0.AGE, t0.COMMENT, t0.FAV, t0.NAME, t0.SURNAME FROM TABLE_A t0 where (t0.ID = ?)");
	}

	@Test
	public void testSimpleSelectDerby() throws Exception {
		setupDerby();
		final QueryVO q = new QueryVO("name", "select ent from TableA ent");
		JPATransformer.transform(q);
		assertThat(q.getError()).isNull();
		assertThat(q.getSQLQuery()).isNotNull().isNotEmpty().hasSize(1);
		assertThat(CoreFactory.getQueries()).isNotEmpty().hasSize(1);
		assertThat(q.getSQLQuery().iterator().next()).isEqualToIgnoringCase("SELECT t0.ID, t0.AGE, t0.COMMENT, t0.FAV, t0.NAME, t0.SURNAME FROM TABLE_A t0");
	}

	@Test
	public void testWithParamDerby() throws Exception {
		setupDerby();
		final QueryVO q = new QueryVO("name", "select ent from TableA ent where ent.id = :id");
		JPATransformer.transform(q);
		assertThat(q.getError()).isNull();
		assertThat(q.getSQLQuery()).isNotNull().isNotEmpty().hasSize(1);
		assertThat(CoreFactory.getQueries()).isNotEmpty().hasSize(1);
		assertThat(q.getSQLQuery().iterator().next()).isEqualToIgnoringCase("SELECT t0.ID, t0.AGE, t0.COMMENT, t0.FAV, t0.NAME, t0.SURNAME FROM TABLE_A t0 where (t0.ID = ?)");
	}

	@Test
	public void testSimpleSelectHSQL() throws Exception {
		setupHSQL();
		final QueryVO q = new QueryVO("name", "select ent from TableA ent");
		JPATransformer.transform(q);
		assertThat(q.getError()).isNull();
		assertThat(q.getSQLQuery()).isNotNull().isNotEmpty().hasSize(1);
		assertThat(CoreFactory.getQueries()).isNotEmpty().hasSize(1);
		assertThat(q.getSQLQuery().iterator().next()).isEqualToIgnoringCase("SELECT t0.ID, t0.AGE, t0.COMMENT, t0.FAV, t0.NAME, t0.SURNAME FROM TABLE_A t0");
	}

	@Test
	public void testWithParamHSQL() throws Exception {
		setupHSQL();
		final QueryVO q = new QueryVO("name", "select ent from TableA ent where ent.id = :id");
		JPATransformer.transform(q);
		assertThat(q.getError()).isNull();
		assertThat(q.getSQLQuery()).isNotNull().isNotEmpty().hasSize(1);
		assertThat(CoreFactory.getQueries()).isNotEmpty().hasSize(1);
		assertThat(q.getSQLQuery().iterator().next()).isEqualToIgnoringCase("SELECT t0.ID, t0.AGE, t0.COMMENT, t0.FAV, t0.NAME, t0.SURNAME FROM TABLE_A t0 where (t0.ID = ?)");
	}

}
